if select(2, UnitClass("player")) ~= "PRIEST" then return end

local name = ...

local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local shadowSpec = SPEC_PRIEST_SHADOW

local VilinkasInsanity = _G["VilinkasInsanity"]
local insanityBar = VilinkasInsanity.InsanityBar

local function GetPositionRelToCenter(point, xOffset, yOffset)
	local halfWidth = UIParent:GetWidth() / 2
	local halfHeight = UIParent:GetHeight() / 2
	if point == "BOTTOMLEFT" then
		xOffset = xOffset - halfWidth
		yOffset = yOffset - halfHeight
	elseif point == "BOTTOM" then
		--xOffset = xOffset
		yOffset = yOffset - halfHeight
	elseif point == "BOTTOMRIGHT" then
		xOffset = halfWidth - xOffset
		yOffset = yOffset - halfHeight
	elseif point == "RIGHT" then
		xOffset = halfWidth - xOffset
		--yOffset = yOffset
	elseif point == "TOPRIGHT" then
		xOffset = halfWidth - xOffset
		yOffset = halfHeight - yOffset
	elseif point == "TOP" then
		--xOffset = xOffset
		yOffset = halfHeight - yOffset
	elseif point == "TOPLEFT" then
		xOffset = xOffset - halfWidth
		yOffset = halfHeight - yOffset
	elseif point == "LEFT" then
		xOffset = xOffset - halfWidth
		--yOffset = yOffset
	end
	
	return xOffset, yOffset
end

local function CreateOptions()
	local LSM = LibStub("LibSharedMedia-3.0")
	
	local function get(info)
		local category, option = string.split(".", info.arg)
		--print("Set: category: " .. category .. " option: ".. option)
		local val = insanityBar.db.profile[category][option]
		if type(val) == "table" then
			return unpack(val)
		else
			return val
		end
	end
	
	local function set(info, ...)
		local category, option = string.split(".", info.arg)
		--print("Set: category: " .. category .. " option: ".. option)
		if select(2, ...) then
			local value = {}
			value[1] = select(1, ...)
			value[2] = select(2, ...)
			value[3] = select(3, ...)
			value[4] = select(4, ...)
			insanityBar.db.profile[category][option] = value
		else
			insanityBar.db.profile[category][option] = select(1, ...)
		end
		
		if option == "orientation" then
			local width = insanityBar.db.profile.general.width
			local height = insanityBar.db.profile.general.height
			insanityBar.db.profile.general.width = height
			insanityBar.db.profile.general.height = width
		end
		
		insanityBar:Setup()
	end
	
	local function clearSTMReports()
		insanityBar:ClearSavedSTMReports()
	end
	
	local options = {
		type = "group",
		childGroups = "tab",
		get = get,
		set = set,
		args = {
			displayGroup = {
				type = "group",
				name = "General",
				order = 1,
				args = {
					orientationheader = {
						type = "header",
						name = "Orientation",
						order = 1
					},
					orientationdisc = {
						type = "description",
						name = "Experimental!",
						order = 2
					},
					orientationselect = {
						type = 'select',
						name = "",
						order = 3,
						values = { 
							["HORIZONTAL"] = "Horizontal",
							["VERTICAL"] = "Vertical",
						},
						arg = "general.orientation"
					},
					posheader = {
						type = "header",
						name = "Position",
						order = 4
					},
					posx = {
						type = "range",
						name = "X",
						order = 5,
						softMin = -(UIParent:GetWidth() / 2),
						softMax = UIParent:GetWidth() / 2,
						step = 1,
						arg = "general.posx"
					},
					posy = {
						type = "range",
						name = "Y",
						order = 5,
						softMin = -(UIParent:GetHeight() / 2),
						softMax = UIParent:GetHeight() / 2,
						step = 1,
						arg = "general.posy"
					},
					sizeheader = {
						type = "header",
						name = "Size",
						order = 6
					},
					width = {
						type = "range",
						name = "Width",
						order = 7,
						softMin = 10,
						softMax = 500,
						step = 1,
						arg = "general.width"
					},
					height = {
						type = "range",
						name = "Height",
						order = 7,
						softMin = 10,
						softMax = 500,
						step = 1,
						arg = "general.height"
					},
					oocheader = {
						type = "header",
						name = "Out of combat",
						order = 8
					},
					alphaooc = {
						type = "range",
						name = "Opacity out of combat",
						arg = "general.alphaooc",
						order = 9,
						min = 0,
						max = 1,
						step = 0.1,
						isPercent = true
					},
					backgroundheader = {
						type = "header",
						name = "Background",
						order = 10
					},
					backgroundcolor = {
						type = "color",
						name = "Color",
						order = 11,
						hasAlpha = true,
						arg = "background.color"
					},
					borderheader = {
						type = "header",
						name = "Border",
						order = 12
					},
					bordercolor = {
						type = "color",
						name = "Color",
						order = 13,
						hasAlpha = true,
						arg = "border.color"
					},
					borderthickness = {
						type = "range",
						name = "Thickness",
						order = 13,
						min = 1,
						max = 5,
						step = 1,
						arg = "border.thickness"
					},
					vfmarkheader = {
						type = "header",
						name = "Enter voidform mark",
						order = 14
					},
					vfmarkcolor = {
						type = "color",
						name = "Color",
						order = 15,
						hasAlpha = true,
						arg = "overlay.color"
					},
					vfmarkthickness = {
						type = "range",
						name = "Thickness",
						order = 15,
						min = 1,
						max = 5,
						step = 1,
						arg = "overlay.thickness"
					},
				}
			},
			bars = {
				type = "group",
				name = "Bars",
				order = 2,
				args = {
					textureselect = {
						type = 'select',
						dialogControl = 'LSM30_Statusbar',
						order = 1,
						name = "Bar texture",
						values = LSM:HashTable("statusbar"),
						arg = "bars.texture"
					},
					insanityheader = {
						type = "header",
						name = "Insanity bar",
						order = 2
					},
					insanitybarcolor = {
						type = "color",
						name = "Color",
						order = 3,
						hasAlpha = false,
						arg = "bars.insbarcolor"
					},
					castbarheader = {
						type = "header",
						name = "Cast bar",
						order = 4
					},
					castbarcolor = {
						type = "color",
						name = "Color",
						order = 5,
						hasAlpha = false,
						arg = "bars.castbarcolor"
					},
					asbarheader = {
						type = "header",
						name = "Auspicious spirits bar",
						order = 6
					},
					asbarcolor = {
						type = "color",
						name = "Color",
						order = 7,
						hasAlpha = false,
						arg = "bars.asbarcolor"
					}
				}
			},
			text = {
				type = "group",
				name = "Text",
				order = 3,
				args = {
					fontfile = {
						type = 'select',
						dialogControl = 'LSM30_Font',
						name = "Font",
						order = 1,
						values = LSM:HashTable("font"),
						arg = "text.file"
					},
					outline = {
						type = 'select',
						name = "Outline",
						order = 1,
						values = { 
							["OUTLINE, MONOCHROME"] = "Monochrome Outline",
							["THICKOUTLINE, MONOCHROME"] = "Monochrome Thick Outline",
							["NONE"] = "None",
							["OUTLINE"] = "Outline",
							["THICKOUTLINE"] = "Thick Outline",
						},
						arg = "text.flag"
					},
					insanitytextheader = {
						type = "header",
						name = "Insanity text",
						order = 2
					},
					insanitytextdisc = {
						type = "description",
						name = "Displays your current insanity value, predicted insanity generated by successful spell cast and insanity generated by auspicious spirits.\nFormat: auspicious spirits + sucessful cast + current insanity",
						order = 3
					},
					insanitytextcolor = {
						type = "color",
						name = "Insanity color",
						order = 4,
						hasAlpha = false,
						arg = "text.insanitycolor"
					},
					casttextcolor = {
						type = "color",
						name = "Cast color",
						order = 5,
						hasAlpha = false,
						arg = "text.castcolor"
					},
					astextcolor = {
						type = "color",
						name = "Auspicious spirits color",
						order = 6,
						hasAlpha = false,
						arg = "text.ascolor"
					},
					insanitytextsize = {
						type = "range",
						name = "Size",
						min = 6,
						max = 72,
						step = 1,
						order = 7,
						arg = "text.insanitysize"
					},
					vfstacktextheader = {
						type = "header",
						name = "Voidform stack and haste text",
						order = 8
					},
					vfstacktextdisc = {
						type = "description",
						name = "Displays current voidform stack and player haste.",
						order = 9
					},
					vfstacktextcolor = {
						type = "color",
						name = "Color",
						order = 10,
						hasAlpha = false,
						arg = "text.vfstackcolor"
					},
					vfstacktextsize = {
						type = "range",
						name = "Size",
						min = 6,
						max = 72,
						step = 1,
						order = 10,
						arg = "text.vfstacksize"
					},
					vftimetextheader = {
						type = "header",
						name = "Voidform time text",
						order = 11
					},
					vftimetextdisc = {
						type = "description",
						name = "Displays your remaining time in voidform.",
						order = 12
					},
					vftimetextcolor = {
						type = "color",
						name = "Color",
						order = 13,
						hasAlpha = false,
						arg = "text.vftimecolor"
					},
					vftimetextsize = {
						type = "range",
						name = "Size",
						min = 6,
						max = 72,
						step = 1,
						order = 13,
						arg = "text.vftimesize"
					},
				}
			},
			animations = {
				type = "group",
				name = "Animations",
				order = 4,
				args = {
					oocheader = {
						type = "header",
						name = "Out of combat animation",
						order = 1
					},
					oocduration = {
						type = "range",
						name = "Duration",
						min = 0,
						softMax = 5,
						step = 0.1,
						order = 2,
						arg = "animations.oocduration"
					},
					oocstartdelay = {
						type = "range",
						name = "Start Delay",
						min = 0,
						softMax = 5,
						step = 0.1,
						order = 2,
						arg = "animations.oocstartdelay"
					},
					pulseheader = {
						type = "header",
						name = "Ready to enter voidform animation",
						order = 3
					},
					pulsecolor = {
						type = "color",
						name = "Color",
						order = 4,
						hasAlpha = false,
						arg = "animations.bordercolor"
					},
					pulseduration = {
						type = "range",
						name = "Duration",
						min = 0,
						softMax = 5,
						step = 0.1,
						order = 4,
						arg = "animations.borderduration"
					},
				}
			},
			misc = {
				type = "group",
				name = "Misc",
				order = 5,
				args = {
					liheader = {
						type = "header",
						name = "Lingering Insanity",
						order = 1
					},
					lidescription = {
						type = "description",
						name = "Choose whether to display Lingering Insanity stacks when this talent is selected.",
						order = 2
					},
					liselect = {
						type = 'select',
						name = "",
						order = 3,
						values = { 
							[1] = "Always",
							[2] = "Only with Void Lord",
							[0] = "Never",
						},
						arg = "misc.lingeringinsanity"
					},
					--[[litoggle = {				-- Patch: 7.1.5
						type = 'toggle',
						name = "Show",
						order = 3,
						arg = "misc.lingeringinsanity"
					},]]--
					lotvheader = {
						type = "header",
						name = "Legacy of The Void",
						order = 4
					},
					lotvdescription = {
						type = "description",
						name = "If Legacy of The Void telent is selected, change at which insanity value 'Ready to enter voidform animation' starts playing.",
						order = 5
					},
					lotvrange = {
						type = "range",
						name = "Insanity",
						min = 70,
						--min = 65,	-- Patch: 7.1.5
						max = 100,
						step = 1,
						order = 6,
						arg = "misc.lotvthreshold"
					},
					stmreportheader = {
						type = "header",
						name = "STM reports",
						order = 7
					},
					stmreportdescription = {
						type = "description",
						name = "At the end of surrender to madness save and display a report. To access saved reports use command |cFF9370DB/vilins stm|r.",
						order = 8
					},
					stmreportselect = {
						type = 'select',
						name = "",
						order = 9,
						values = {
							[0] = "Disable",
							[1] = "Display only",
							[2] = "Save only",
							[3] = "Save and display",
						},
						arg = "misc.stmreportingenabled"
					},
					stmreportarchiverange = {
						type = 'range',
						name = "Maximum number of saved reports",
						--width = "full",
						min = 1,
						max = 5,
						step = 1,
						order = 10,
						arg = "misc.stmreportingmaxsaved"
					},
					stmreportcleararchivebutton = {
						type = "execute",
						name = "Clear saved reports",
						func = clearSTMReports,
						order = 11
					},
					patchheader = {
						type = "header",
						name = "Patch 7.1.5",
						order = 12
					},
					patchdescription = {
						type = "description",
						name = "Enable patch 7.1.5 changes to shadow priest.",
						order = 13
					},
					patchtoggle = {
						type = "toggle",
						name = "Enable",
						order = 14,
						arg = "misc.patch70105enable"
					},
				}
			}
		}
	}
	
	return options
end

function VilinkasInsanityBar_Options_OnUpdate(self)
	local insanity = 40
	local castGainPrediction = 24
	local asGainPrediction = 12
	
	insanityBar:SetInsanityValue(insanity, castGainPrediction, asGainPrediction)
	
	insanityBar:SetVfStackValue(20, 58)
	insanityBar.VoidformTimeFontString:SetText(format("%.1f", 1.5))
end

local function UnlockInsanityBar()
	insanityBar:SetMovable(true)
	insanityBar:UnregisterEvent("UNIT_MAXPOWER", "player")
	insanityBar:UnregisterEvent("PLAYER_REGEN_ENABLED")
	insanityBar:UnregisterEvent("PLAYER_REGEN_DISABLED")
	insanityBar:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	insanityBar:SetScript("OnUpdate", VilinkasInsanityBar_Options_OnUpdate)
	insanityBar.VoidformReadyAnim:Stop()
	insanityBar:Show()
	insanityBar:SetAlpha(1)
end

local function LockInsanityBar()
	insanityBar:SetMovable(false)
	insanityBar:ACTIVE_TALENT_GROUP_CHANGED()
end

VilinkasInsanityOptionsMixin = {}

function VilinkasInsanityOptionsMixin:OnLoad()
	local container = LibStub("AceGUI-3.0"):Create("SimpleGroup")
	container.frame:SetParent(self)
	container.frame:SetPoint("TOPLEFT", self, "TOPLEFT", 17, -20)
	container.frame:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -17, 12)
	container.frame:Show()
	self.container = container
	
	local options = CreateOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("VilinkasInsanityOptions", options)
	
	self:Setup()
end

function VilinkasInsanityOptionsMixin:OnEvent(event, ...)
	if event == "ADDON_LOADED" then
		if name == ...	then
			local xOffset = GetScreenWidth() / 10
			local yOffset = GetScreenHeight() - (GetScreenHeight() / 10)
			VilinkasInsanityOptionsDB = VilinkasInsanityOptionsDB or {}
			self.db = VilinkasInsanityOptionsDB
			if self.db.xOffset == nil then
				self.db.xOffset = xOffset
			end
			if self.db.yOffset == nil then
				self.db.yOffset = yOffset
			end
			self:UpdateOptions()
		end
	elseif event == "PLAYER_TALENT_UPDATE" then
		self:Setup()
	elseif event == "PLAYER_REGEN_ENABLED" then
		if self.showWhenOutOfCombat then
			self:Show()
			self.showWhenOutOfCombat = false
		end
	elseif event == "PLAYER_REGEN_DISABLED" then
		if self:IsShown() then
			self:Hide()
			self.showWhenOutOfCombat = true
		end
	end
end

function VilinkasInsanityOptionsMixin:OnShow()
	UnlockInsanityBar()
	
	self.VilinkasInsanityBarMover:AnchorToPowerFrame()
end

function VilinkasInsanityOptionsMixin:OnHide()
	LockInsanityBar()
end

function VilinkasInsanityOptionsMixin:Setup()
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	
	local playerSpec = GetSpecialization()
	if playerSpec == shadowSpec then
		AceConfigDialog:Open("VilinkasInsanityOptions", self.container)
		AceConfigDialog:Open("VilinkasInsanityOptions", self.container)
			
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
			
	else
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		self:UnregisterEvent("PLAYER_REGEN_DISABLED")

	end
end

function VilinkasInsanityOptionsMixin:SaveOptionsFrameChanges()
	local xOffset = self:GetLeft()
	local yOffset = self:GetBottom() + self:GetHeight()
	if self.Header:GetLeft() < 0 then
		xOffset = xOffset - self.Header:GetLeft()
	elseif self.Header:GetRight() > GetScreenWidth()  then
		xOffset = xOffset - (self.Header:GetRight() - GetScreenWidth())
	end
	if self.Header:GetBottom() < 0 then
		yOffset = yOffset - self.Header:GetBottom()
	elseif self.Header:GetTop() > GetScreenHeight() then
		yOffset = yOffset - (self.Header:GetTop() - GetScreenHeight())
	end
	self.db.xOffset = xOffset
	self.db.yOffset = yOffset
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", xOffset, yOffset)
end

function VilinkasInsanityOptionsMixin:UpdateOptions()
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.xOffset, self.db.yOffset)
end

function VilinkasInsanityOptionsMixin:UnlockPowerFrame(powerFrame)
	powerFrame:UnregisterEvent("UNIT_POWER_FREQUENT", "player")
	powerFrame:UnregisterEvent("UNIT_MAXPOWER", "player")
	powerFrame:UnregisterEvent("PLAYER_REGEN_ENABLED")
	powerFrame:UnregisterEvent("PLAYER_REGEN_DISABLED")
end

function VilinkasInsanityOptionsMixin:LockPowerFrame(powerFrame)
	powerFrame:RegisterEvent("UNIT_POWER_FREQUENT", "player")
	powerFrame:RegisterEvent("UNIT_MAXPOWER", "player")
	powerFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	powerFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
end

VilinkasInsanityBarMoverMixin = {}

function VilinkasInsanityBarMoverMixin:OnMouseDown()
	insanityBar:StartMoving()
end

function VilinkasInsanityBarMoverMixin:OnMouseUp()
	local parent = self:GetParent()
	insanityBar:StopMovingOrSizing()
	local point, _, _, xOffset, yOffset = insanityBar:GetPoint(1)
	xOffset, yOffset = GetPositionRelToCenter(point, xOffset, yOffset)
	insanityBar.db.profile.general.posx = xOffset
	insanityBar.db.profile.general.posy = yOffset
	AceConfigDialog:Open("VilinkasInsanityOptions", parent.container)
end

function VilinkasInsanityBarMoverMixin:AnchorToPowerFrame()
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", insanityBar, "TOPLEFT")
	self:SetPoint("BOTTOMRIGHT", insanityBar, "BOTTOMRIGHT")
	self.Text:SetText("Drag to move")
end
